/////////////////////////////////////////////////
// filename.h
//
// Copyright 1994, Scott Wood, CCP
//
// File Name class.
//   Encapsulates filename pathing functions for
//   parsing and path maintenance.
//
/////////////////////////////////////////////////

#ifndef __FILENAME_H
  #define __FILENAME_H

  #ifndef __CSTRING_H
    #include <cstring.h>
    #endif
  #ifndef __IOSTREAM_H
    #include <iostream.h>
    #endif
  #ifndef __BOOLEAN_H
    #include <boolean.h>
    #endif
  #ifndef __DIR_H
    #include <dir.h>
    #endif
    
  // class FileName
  class _PTCLASS FileName
    {
      public:

        FileName (void);
        FileName (const char _FAR * sourceString);
        FileName (const string _FAR & sourceString);

        unsigned Hash (void) const;

        int Compare (const FileName _FAR &) const;

        // path info
        string Path  (void) const;

        string Drive (void) const;
        string Dir   (void) const;
        string Name  (void) const;
        string Ext   (void) const;

        string DriveDir (void) const;
        string NameExt  (void) const;
        string DriveDirName  (void) const;

        void SetDrive (const string _FAR & a_drive);
        void SetDir   (const string _FAR & a_dir);
        void SetName  (const string _FAR & a_name);
        void SetExt   (const string _FAR & a_ext);

        void SetDriveDir (const string _FAR & a_driveDir);
        void SetNameExt  (const string _FAR & a_nameExt);

        void SetDriveDirToCurrent (void);
        void SetDriveDirToEXE (void);
        void SetPathToEXE (void);

        boolean DriveDirExists (void) const;
        boolean FileExists (void) const;

        boolean Remove (void);

        boolean MakeDir (void);
        boolean RemoveDir (void);
        
        friend ostream _FAR & _PTFUNC operator << (ostream _FAR & str, FileName _FAR &);

        static int MaxPathLength  (void);
        static int MaxDriveLength (void);
        static int MaxDirLength   (void);
        static int MaxNameLength  (void);
        static int MaxExtLength   (void);

        friend int operator == (const FileName _FAR & l, const FileName _FAR & r);
        friend int operator != (const FileName _FAR & l, const FileName _FAR & r);
        friend int operator <  (const FileName _FAR & l, const FileName _FAR & r);
        friend int operator >  (const FileName _FAR & l, const FileName _FAR & r);
        friend int operator <= (const FileName _FAR & l, const FileName _FAR & r);
        friend int operator >= (const FileName _FAR & l, const FileName _FAR & r);

      protected:

        string pDrive;
        string pDir;
        string pName;
        string pExt;

        void pBuild (const char _FAR * sourceString);
    };

  inline string FileName::Drive (void) const
    {
      return pDrive;
    };

  inline string FileName::Dir   (void) const
    {
      return pDir;
    };

  inline string FileName::Name  (void) const
    {
      return pName;
    };

  inline string FileName::Ext   (void) const
    {
      return pExt;
    };

  inline int FileName::MaxPathLength (void)
    {
      return MAXPATH - 1;
    };

  inline int FileName::MaxDriveLength (void)
    {
      return MAXDRIVE - 1;
    };

  inline int FileName::MaxDirLength (void)
    {
      return MAXDIR - 1;
    };

  inline int FileName::MaxNameLength (void)
    {
      return MAXFILE - 1;
    };

  inline int FileName::MaxExtLength (void)
    {
      return MAXEXT - 1;
    };

  inline int operator == (const FileName _FAR & l, const FileName _FAR & r)
    {
      return l.Compare (r) == 0;
    };

  inline int operator != (const FileName _FAR & l, const FileName _FAR & r)
    {
      return l.Compare (r) != 0;
    };

  inline int operator <  (const FileName _FAR & l, const FileName _FAR & r)
    {
      return l.Compare (r) < 0;
    };

  inline int operator >  (const FileName _FAR & l, const FileName _FAR & r)
    {
      return l.Compare (r) > 0;
    };

  inline int operator <= (const FileName _FAR & l, const FileName _FAR & r)
    {
      return l.Compare (r) <= 0;
    };

  inline int operator >= (const FileName _FAR & l, const FileName _FAR & r)
    {
      return l.Compare (r) >= 0;
    };

#endif

