/////////////////////////////////////////////////////////////////////
//
//  File:    REPORT.H
//
//  Author:  Scott Wood
//
//  Created: 01 Nov 92 RSW
//
//  Revised: 27 Jan 94 RSW
//           29 Mar 94 RSW, ported to Windows from XVT
//           01 Jul 94 RSW, converted to Borland OWL
//
//  Purpose: To provide the Report Class and member functions used to
//           drive Windows printer interface.
//
//  Notes:   See source for detailed description of each function
/////////////////////////////////////////////////////////////////////
#ifndef __REPORT_H
  #define __REPORT_H

  #include <windows.h>
  #include <commdlg.h>
  #include <print.h>

  #ifndef __PTDEFS
	 #include <ptdefs.h>
	 #endif
  #ifndef __BOOLEAN_H
	 #include <boolean.h>
	 #endif
  #ifndef __FILENAME_H
	 #include <filename.h>
	 #endif
  #ifndef __CSTRING_H
	 #include <cstring.h>
	 #endif
  #ifndef __RPTSETUP_H
	 #include <rptsetup.h>
	 #endif
  #ifndef __RPTABORT_H
	 #include <rptabort.h>
	 #endif

  enum paperOrientEnum
    {
      paperOrientDefault,
      paperOrientPortrait,
      paperOrientLandscape,
	 };

  class _PTCLASS Report
    {
      public:

        Report (void);
       ~Report (void);

		  boolean Open (boolean doSetup, const string _FAR & title);

		  void    Close (void);

		  boolean SetupReport (void);

		  boolean SetupPrinter (void);

		  boolean RedirectOutput (reportOutputEnum where, const FileName _FAR & name);

		  boolean BeginReport (const string _FAR & reportName);
		  void    EndReport (void);

		  void PageBreak (void);

		  void SetPageHeader (void (*func)(Report _FAR *, void _FAR *), void _FAR * name);
		  void SetPageFooter (void (*func)(Report _FAR *, void _FAR *), void _FAR * name);

		  void SetNumberOfCopies (int);

        int X (void);
        int Y (void);
        void SetX (int);
        void SetY (int);

        int PageNumber (void);
        int LineHeight (void);
        int PageHeight (void);
        int PageWidth (void);
        int CharWidth (void);
        int HRes (void);
        int VRes (void);

        int  CharPerLine (void);
        void SetCharPerLine (int newVal);

        int Line (void);
        void SetLine (int);

        TRect MarginRct (void);
        void  SetMarginRct (TRect _FAR & rct);

        TRect BorderRct (void);
        void  SetBorderRct (TRect _FAR & rct);

        void ChooseFont (void);

		  void GetFont (string _FAR & typeFace, int& height, int& width, int& weight, boolean& italic);

		  void SetFont (const string _FAR & typeFace, int height, int width, int weight, boolean italic);
        void SetFontHeight (int height);
        void SetFontWeight (int weight);
        void SetFontItalic (boolean italic);

		  int printf (const char _FAR * fmt, ...);

		  int PrintLeftJustify (const char _FAR *str);
		  int PrintRightJustify (const char _FAR *str);
		  int PrintCenterJustify (const char _FAR *str);

		  void PrintNewLine (void);

		  void PrintLine (int length);
		  int  PrintChar (char ch, int length);

		  int PrintJustifyLine (const char _FAR *str);
		  int PrintJustifyParagraph (const char _FAR *str);

		  void PrintDashLine (const char _FAR *, boolean solid);

		  boolean TestPageBreak (int addLines);

		  boolean TestPercentPageLeft (int percent);

		  boolean TestLinesLeft (int lines);
		  int NumLinesAvailable (void);

        int  GetTextWidth (const char _FAR *str);

		  void TextOut (const TPoint _FAR & pt, const char _FAR *str, int len = -1);
		  void MoveTo (const TPoint _FAR & pt);
		  void LineTo (const TPoint _FAR & pt);
        void Rectangle (const TRect _FAR & rct);

        void DrawBitmap (HINSTANCE hInstance, const char _FAR *name, TRect _FAR & rct);

        HDC DC (void);

        TPrintDC _FAR * PDC (void);

        boolean IsAbort (void);

        void SetTestMargin (boolean);

        void OrientPaper (paperOrientEnum);

      private:

		  void pSetFont (void);

        void pStartPage (void);
        void pEndPage (void);

        void pPutMargin (void);

        void pSetMapping (void);

		  string pTitle;

		  boolean pInReport;

		  boolean pTestPage;
		  boolean pFirstPage;
		  boolean pLastPage;
		  boolean pTestMargin;

        TPrintDC *pHDC;

		  DlgReportSetup::ReportData *pData;

        int pX;
        int pY;

        int pPageno;

        int pLineHt;
        int pCharWd;
        int pPageWd;
        int pPageHt;
        int pVRes;
        int pHRes;

        int pCharPerLine;

		  TRect pMarginRct;
		  TRect pBorderRct;

		  int pOutputChno;

		  void (*pHdrFunc)(Report *, void *);
        void (*pFtrFunc)(Report *, void *);

		  void *pHdrData;
		  void *pFtrData;

		  LOGFONT pLogFont;

		  static BOOL FAR PASCAL pReportAbortProc (HDC, int);

		  static DlgReportAbort *pAbortWin;
	 };

  inline boolean Report::IsAbort (void)
	 {
		return pAbortWin->IsAbort ();
	 };

  inline void Report::SetTestMargin (boolean newVal)
    {
      pTestMargin = newVal;
    };

  inline int Report::LineHeight (void)
    {
      return pLineHt;
    };

  inline int Report::CharWidth (void)
    {
      return pCharWd;
    };

  inline int Report::PageWidth (void)
    {
      return pPageWd;
    };

  inline int Report::PageHeight (void)
    {
      return pPageHt;
    };

  inline int Report::VRes (void)
    {
      return pVRes;
    };

  inline int Report::HRes (void)
    {
      return pHRes;
    };

  inline int Report::CharPerLine (void)
    {
    	 return pCharPerLine;
    };

  inline TRect Report::MarginRct (void)
    {
      return pMarginRct;
    };

  inline TRect Report::BorderRct (void)
    {
      return pBorderRct;
    };

  inline int Report::PageNumber (void)
    {
      return pPageno;
    }

#endif

