#ifndef __RPTPRINT_H
  #define __RPTPRINT_H

  #include <owl\owlpch.h>

  #ifndef __RPTABORT_H
    #include <rptabort.h>
    #endif
  #ifndef __RPTSETUP_H
    #include <rptsetup.h>
    #endif

  _PTDEF (Report);

  enum paperOrientEnum
    {
      paperOrientDefault,
      paperOrientPortrait,
      paperOrientLandscape,
    };

  class _PTCLASS ReportPrinter
    {
      public:

        ReportPrinter (void);
       ~ReportPrinter (void);

        boolean SetupReport (TWindow *parent);
        boolean SetupPrinter (TWindow *parent);

        boolean OpenPrinter (TWindow *parent, Report *rpt, boolean doSetup);
        boolean ClosePrinter (void);

        void RedirectOutput (reportOutputEnum where, const FileName _FAR &);
        void OrientPaper (paperOrientEnum orientation);
        void SetCopies (int copies);

        boolean PrintReport (TWindow *parent, Report *rpt, boolean doSetup);

        void PageBreak (void);

        int  X (void);
        int  Y (void);
        void SetX (int);
        void SetY (int);

        int  PageNumber (void);
        int  LineHeight (void);
        int  PageHeight (void);
        int  PageWidth (void);
        int  CharWidth (void);
        int  HorzRes (void);
        int  VertRes (void);

        int  Line (void);
        void SetLine (int);

        Rct  MarginRct (void);
        void SetMarginRct (Rct _FAR &);

        Rct  BorderRct (void);
        void SetBorderRct (Rct _FAR &);

        void ChooseFont (void);
        void GetFont (char _FAR *typeFace, int _FAR & height,
                      int _FAR & width, int _FAR &weight, boolean _FAR & italic);
        void SetFont (const char _FAR *typeFace, int height, int width,
                      int weight, boolean italic);
        void SetFontHeight (int height);
        void SetFontWeight (int weight);
        void SetFontItalic (boolean italic);

        int  printf (const char _FAR *fmt, ...);
        int  Printf (const char _FAR *fmt, ...);
        int  PrintLeftJustify (const char _FAR *);
        int  PrintRightJustify (const char _FAR *);
        int  PrintCenterJustify (const char _FAR *);

        void NewLine (void);

        int  PrintLine (int length);
        int  PrintChar (char ch, int length);

        int  PrintJustifyLine (const char _FAR *);
        int  PrintJustifyParagraph (const char _FAR *);

        void PrintDashLine (const char _FAR *, boolean solid);

        boolean TestPageBreak (int lines);
        boolean TestPercentPageLeft (int percent);

        boolean TestLinesLeft (int lines);
        int NumLinesAvailable (void);

        void TextOut (Coordinate _FAR &, const char _FAR *, int len);
        void MoveTo (Coordinate _FAR &);
        void LineTo (Coordinate _FAR &);
        void Rectangle (Rct _FAR &);

        void DrawBitMap (HINSTANCE hInstance, const char _FAR *name, Rct _FAR &rct);

        HDC  hDC (void);

        boolean IsAbort (void);

        void SetTestMargin (boolean);

      protected:

        void GetDefaultPrinter (void);

        boolean DoSetup (TWindow *parent);

      private:

        TWindow *pParent;

        Report *pRpt;

        int pOpenCnt;

        DlgReportSetup::ReportData pData;

        TPrintDC *pHDC;

        boolean pTestPage;
        boolean pFirstPage;
        boolean pLastPage;
        boolean pTestMargin;

        long pX;
        long pY;

        long pPagewd;
        long pPageht;
        long pHorzRes;
        long pVertRes;

        int pPageno;
        int pLineht;
        int pCharwd;

        Rct pMarginRct;
        Rct pBorderRct;

        int pOutputChno;

        LOGFONT pLogFont;

        int pPrint (const char _FAR *);

        void pSetFont (void);

        void pBeginReport (void);
        void pEndReport (void);

        void pStartPage (void);
        void pEndPage (void);

        void pPutMargin (void);

        static BOOL FAR PASCAL reportAbortProc (HDC, int);

        static DlgReportAbort *pAbortWin;

    };

  ReportPrinter& _PTFUNC PTPrinter (void);

#endif

