/////////////////////////////////////////////////
// fnutil.cpp
//
// Copyright 1994, Scott Wood, CCP
//
// FileName utilities
//
/////////////////////////////////////////////////

#include <dir.h>
#include <direct.h>
#include <io.h>
#include <dos.h>
#include <fstream.h>
#include <windows.h>
#include <filename.h>

string FileName::DriveDir (void) const
  {
    return pDrive + pDir;
  };

string FileName::DriveDirName (void) const
  {
    return pDrive + pDir + pName;
  };

string FileName::NameExt (void) const
  {
    return pName + pExt;
  };

string FileName::Path (void) const
  {
    return pDrive + pDir + pName + pExt;
  };

boolean FileName::DriveDirExists (void) const
  {
#if defined (__WIN32__)

    string test = pDrive + pDir;
    if (test.substr (test.length ()-1, 1) == "\\")
      test = test.substr (0, test.length () - 1);
    return GetFileAttributes (test.c_str ()) != 0xffffffff;

#else

    struct fcb blk;

    string test = pDrive + pDir;
    if (test.is_null ())      // local directory
      return true;

    return parsfnm (test.c_str (), &blk, 1) != NULL;
#endif
  };

boolean FileName::FileExists (void) const
  {
    return access (Path ().c_str (), 0) == 0;
  };

void FileName::SetDir (const string _FAR & a_dir)
  {
    if (!a_dir.is_null ())
      {
        // truncate to fit, add trailing '\\'
        pDir = a_dir;

        if (pDir.length() > (MAXDIR - 2))
            // MAXDIR includes null char, plus want to leave room for '\\'
            // to be added below
            pDir = pDir.substr (0, MAXDIR - 2);

        if (pDir.substr (pDir.length ()-1, 1) != "\\")
            pDir += "\\";
      }
    else
      pDir = (char)0;
  };

void FileName::SetDrive (const string _FAR & a_drive)
  {
    // in case something is wrong,
    // make up a drive from the first character

    if (!a_drive.is_null ())
        pDrive = a_drive.substr (0, 1) + ":";
    else
        pDrive = (char)0;
  };

void FileName::SetDriveDir (const string _FAR & a_driveDir)
  {
    if (!a_driveDir.is_null ())
      {
        string tmp = a_driveDir;

        if (tmp.substr (tmp.length () - 1, 1) != "\\")
            tmp += "\\";

        char drive [MAXDRIVE];
        char dir   [MAXDIR];

        fnsplit (tmp.c_str (), drive, dir, 0, 0);

        pDrive = drive;
        pDir   = dir;
      }
    else
      {
        pDrive = (char)0;
        pDir   = (char)0;
      };
  };

void FileName::SetExt (const string _FAR & a_ext)
  {
    pExt = a_ext;

    if (!pExt.is_null ())
      {
        if (pExt.substr (0, 1) != ".")
            pExt = "." + pExt;

        if (pExt.length() > MAXEXT - 1)
            pExt = pExt.substr (0, MAXEXT - 1);
      };
  };

void FileName::SetName (const string _FAR & a_name)
  {
    pName = a_name;

    if (pName.length() > MAXFILE - 1)
        pName = pName.substr (0, MAXFILE - 1);
  };

void FileName::SetNameExt (const string _FAR & a_nameExt)
  {
    if (!a_nameExt.is_null ())
      {
        char name [MAXFILE];
        char ext  [MAXEXT];

        fnsplit (a_nameExt.c_str (), 0, 0, name, ext);

        pName  = name;
        pExt   = ext;
      }
    else
      {
        pName = (char)0;
        pExt  = (char)0;
      };
  };

void FileName::SetDriveDirToCurrent (void)
  {
    char bfr [MAXPATH];

    getcwd (bfr, MAXPATH);

    string cwd = bfr; // a little redundant, needed for non-dynamic strings

    // could test for error from getcwd here
    if (cwd.substr (cwd.length () - 1, 1) != "\\")
        cwd += "\\";
    else
      {
        // sometimes get two backslash chars from getcwd
        if (  cwd[cwd.length()-2] == '\\' )
          cwd = cwd.substr (0, cwd.length()-1);
      };

    char drive [MAXDRIVE];
    char dir   [MAXDIR];

    fnsplit (cwd.c_str (), drive, dir, 0, 0);

    pDrive = drive;
    pDir   = dir;
  };

void FileName::SetDriveDirToEXE (void)
  {
    char buf[80];

#if defined (__WIN32__)
    GetModuleFileName (0, buf, sizeof(buf));
#else
    GetModuleFileName(*((HINSTANCE *)MK_FP(GetCurrentTask(), 0x1c)), buf, sizeof(buf));
#endif
    string exePath (buf);

    if (!exePath.is_null ()) // could be null for DOS prior to 3.0
      {
        char drive [MAXDRIVE];
        char dir   [MAXDIR];

        fnsplit (exePath.c_str (), drive, dir, 0, 0);

        pDrive = drive;
        pDir   = dir;
      }
    else
      {
        pDrive = (char)0;
        pDir   = (char)0;
      };
  };

void FileName::SetPathToEXE (void)
  {
    char buf[80];

#if defined (__WIN32__)
    GetModuleFileName (0, buf, sizeof(buf));
#else
    GetModuleFileName(*((HINSTANCE *)MK_FP(GetCurrentTask(), 0x1c)), buf, sizeof(buf));
#endif
    string exePath (buf);

    if (!exePath.is_null ()) // could be null for DOS prior to 3.0
      {
        char drive [MAXDRIVE];
        char dir   [MAXDIR];
        char name  [MAXFILE];
        char ext   [MAXEXT];

        fnsplit (exePath.c_str (), drive, dir, name, ext);

        pDrive = drive;
        pDir   = dir;
        pName  = name;
        pExt   = ext;
      }
    else
      {
        pDrive = (char)0;
        pDir   = (char)0;
        pName  = (char)0;
        pExt   = (char)0;
      };
  };

boolean FileName::Remove (void)
  {
    return (unlink (Path ().c_str ()) == 0);
  };

boolean FileName::MakeDir (void)
  {
    return (mkdir (Path ().c_str ()) == 0);
  };

boolean FileName::RemoveDir (void)
  {
    return (rmdir (Path ().c_str ()) == 0);
  };

