#include <stdio.h>
#include <rptsetup.h>
#include <rptlib.h>
#include <rptlib.rh>
#include <cntrwin.h>

DEFINE_RESPONSE_TABLE1 (DlgReportSetup, TDialog)
  EV_BN_CLICKED (IDOK, CmOk),
  EV_BN_CLICKED (IDC_RPTSETUP_PB_SELECT,  BtnSelect),
  EV_BN_CLICKED (IDC_RPTSETUP_PB_BROWSE,  BtnBrowse),
  EV_BN_CLICKED (IDC_RPTSETUP_RB_PRINTER, BtnPrinter),
  EV_BN_CLICKED (IDC_RPTSETUP_RB_FILE,    BtnFile),
END_RESPONSE_TABLE;

DlgReportSetup::DlgReportSetup (TWindow *parent, ReportData _FAR & rd)
    : TDialog (parent, IDD_RPTSETUP, ResMod)
	 , pData (rd)
	 , pOutput (rd.ReportOutput)
  {
    memset (&pd, 0, sizeof(PRINTDLG));
    pd.lStructSize = sizeof(PRINTDLG);
    pd.hwndOwner = Parent ? Parent->HWindow : 0;
    pd.hInstance = *GetModule();
    pd.Flags = PD_ENABLEPRINTHOOK | PD_ENABLESETUPHOOK | pData.Flags;
    pd.Flags &= ~PD_RETURNDEFAULT;
    pd.Flags &= ~PD_ENABLEPRINTTEMPLATE;
    pd.Flags &= ~PD_ENABLESETUPTEMPLATE;
    pd.lpfnPrintHook = 0;
    pd.lpfnSetupHook = 0;

    pd.nFromPage = (WORD)pData.FromPage;
    pd.nToPage   = (WORD)pData.ToPage;
    pd.nMinPage  = (WORD)pData.MinPage;
    pd.nMaxPage  = (WORD)pData.MaxPage;
    pd.nCopies   = (WORD)pData.Copies;

	 pEditFile  = new TEdit (this, IDC_RPTSETUP_EDIT_FILE);
	 pBtnSelect = new TButton (this, IDC_RPTSETUP_PB_SELECT);
	 pBtnBrowse = new TButton (this, IDC_RPTSETUP_PB_BROWSE);

	 pEditLMargin = new PTEditInch (this, IDC_RPTSETUP_EDIT_LMARGIN, (float)pData.Margins.left / 100.0, 3);
	 pEditTMargin = new PTEditInch (this, IDC_RPTSETUP_EDIT_TMARGIN, (float)pData.Margins.top / 100.0, 3);
	 pEditRMargin = new PTEditInch (this, IDC_RPTSETUP_EDIT_RMARGIN, (float)pData.Margins.right / 100.0, 3);
	 pEditBMargin = new PTEditInch (this, IDC_RPTSETUP_EDIT_BMARGIN, (float)pData.Margins.bottom / 100.0, 3);

    pEditCopies  = new TEdit (this, IDC_RPTSETUP_EDIT_COPIES, 3);
  };

void DlgReportSetup::SetupWindow (void)
  {
    TDialog::SetupWindow ();

    CenterWindow (HWindow);
    pShowOutput ();

    char buf[80];
	 sprintf (buf, "%s on %s", pData.GetDeviceName (), pData.GetOutputName ());
	 SetDlgItemText (IDC_RPTSETUP_TEXT_PRINTER, buf);

    sprintf (buf, "%d", pData.Copies);
    pEditCopies->SetText (buf);
  };

void DlgReportSetup::CmOk (void)
  {
	 pData.ReportOutput = pOutput;

	 char buf[80];
	 pEditFile->GetText (buf, sizeof(buf));
	 pData.OutputFile = buf;

	 pData.Margins.left   = (long)(pEditLMargin->GetValue () * 100);
	 pData.Margins.top    = (long)(pEditTMargin->GetValue () * 100);
	 pData.Margins.right  = (long)(pEditRMargin->GetValue () * 100);
	 pData.Margins.bottom = (long)(pEditBMargin->GetValue () * 100);

    pEditCopies->GetText (buf, sizeof(buf));
    pData.Copies = atoi (buf);

    TDialog::CmOk ();
  };

void DlgReportSetup::BtnSelect (void)
  {
    pd.Flags = PD_PRINTSETUP;
    if (DoSetupData ())
		{
		  char buf[80];

		  sprintf (buf, "%s on %s", pData.GetDeviceName (), pData.GetOutputName ());
		  SetDlgItemText (IDC_RPTSETUP_TEXT_PRINTER, buf);
		};

    pData.Flags &= ~PD_PRINTSETUP;
  };

void DlgReportSetup::BtnBrowse (void)
  {
	 OPENFILENAME ofn;

	 char file[100];

	 GetDlgItemText (IDC_RPTSETUP_EDIT_FILE, file, sizeof(file));
	 FileName fileName (file);
	 fileName.NameExt ().copy (file, sizeof(file));

	 char filter[20];
	 memset (filter, 0, sizeof(filter));
	 strcpy (filter, "*.*");

	 memset (&ofn, 0, sizeof(OPENFILENAME));
	 ofn.lStructSize  = sizeof(OPENFILENAME);
	 ofn.hwndOwner    = HWindow;
	 ofn.lpstrFilter  = filter;
	 ofn.nFilterIndex = 1;
	 ofn.lpstrFile    = file;
	 ofn.nMaxFile     = sizeof(file);
 	 ofn.lpstrTitle   = "Browse";
	 ofn.Flags = OFN_SHOWHELP | OFN_PATHMUSTEXIST;
	 if (GetOpenFileName (&ofn))
		SetDlgItemText (IDC_RPTSETUP_EDIT_FILE, ofn.lpstrFile);
  };

void DlgReportSetup::BtnPrinter (void)
  {
	 pOutput = reportOutputPrinter;
	 pShowOutput ();
  };

void DlgReportSetup::BtnFile (void)
  {
	 pOutput = reportOutputFile;
	 pShowOutput ();
  };

void DlgReportSetup::pShowOutput (void)
  {
	 switch (pOutput)
		{
		  default:
			 pOutput = reportOutputPrinter;
		  case reportOutputPrinter:
			 pEditFile->EnableWindow (false);
			 pBtnBrowse->EnableWindow (false);
			 pBtnSelect->EnableWindow (true);
			 CheckRadioButton (IDC_RPTSETUP_RB_PRINTER, IDC_RPTSETUP_RB_FILE,
									 IDC_RPTSETUP_RB_PRINTER);
			 break;

		  case reportOutputFile:
			 pEditFile->EnableWindow (true);
			 pBtnBrowse->EnableWindow (true);
			 pBtnSelect->EnableWindow (false);
			 CheckRadioButton (IDC_RPTSETUP_RB_PRINTER, IDC_RPTSETUP_RB_FILE,
									 IDC_RPTSETUP_RB_FILE);
			 break;
		};
  };

int DlgReportSetup::DoSetupData ()
  {
    pData.Unlock ();
    pd.hDevMode  = pData.hDevMode;
    pd.hDevNames = pData.hDevNames;
    int ret = ::PrintDlg (&pd);
    if (ret)
      {
        pData.Flags = pd.Flags;
        pData.Error = 0;
        pData.hDC   = pd.hDC;
        pData.FromPage = pd.nFromPage;
        pData.ToPage = pd.nToPage;
        pData.Copies = pd.nCopies;
      }
    else
      pData.Error = ::CommDlgExtendedError ();

    pData.hDevMode  = pd.hDevMode;
    pData.hDevNames = pd.hDevNames;
    pData.Lock();

    return ret ? IDOK : IDCANCEL;
  };

BOOL DlgReportSetup::GetDefaultPrinter ()
  {
    pd.Flags |= PD_RETURNDEFAULT;
    pData.ClearDevMode ();
    pData.ClearDevNames();
    return DoSetupData () == IDOK;
  };

DlgReportSetup::ReportData::ReportData (void)
	 : Flags (PD_ALLPAGES | PD_COLLATE)
    , Error (0)
    , FromPage (-1)
    , ToPage (-1)
    , MinPage (-1)
    , MaxPage (-1)
    , Copies (1)
    , hDevMode (0)
    , DevMode (0)
    , hDevNames (0)
    , DevNames (0)
    , hDC (0)
	 , ReportOutput (reportOutputPrinter)
  {
	 memset (&Margins, 0, sizeof (Margins));
  };

DlgReportSetup::ReportData::~ReportData (void)
  {
    if (hDevMode)
      {
        ::GlobalUnlock (hDevMode);
        ::GlobalFree (hDevMode);
      };

    if (hDevNames)
      {
        ::GlobalUnlock (hDevNames);
        ::GlobalFree (hDevNames);
      };

    if (hDC)
      ::DeleteDC (hDC);
  };

void DlgReportSetup::ReportData::Lock ()
  {
    if (hDevMode)
      DevMode = (DEVMODE far*)::GlobalLock (hDevMode);
    else
      DevMode = 0;
    if (hDevNames)
      DevNames = (DEVNAMES far*)::GlobalLock (hDevNames);
    else
      DevNames = 0;
  };

void DlgReportSetup::ReportData::Unlock ()
  {
    if (hDevMode)
      {
        ::GlobalUnlock (hDevMode);
        DevMode = 0;
      }
    if (hDevNames)
      {
        ::GlobalUnlock (hDevNames);
        DevNames = 0;
      }
    if (hDC)
      {
        ::DeleteDC (hDC);
        hDC = 0;
      }
  }

void DlgReportSetup::ReportData::ClearDevMode ()
  {
    if (hDevMode)
      {
        ::GlobalUnlock(hDevMode);
        ::GlobalFree(hDevMode);
        hDevMode = 0;
        DevMode = 0;
      }
  }

void DlgReportSetup::ReportData::SetDevMode(const DEVMODE far* devMode)
  {
    ClearDevMode();
    if (devMode)
      {
        int size = sizeof(DEVMODE) + devMode->dmDriverExtra;
        hDevMode = ::GlobalAlloc(GHND, size);
        DevMode = (DEVMODE far*)::GlobalLock(hDevMode);
        memcpy(DevMode, devMode, size);
      }
  }

void DlgReportSetup::ReportData::ClearDevNames()
  {
    if (hDevNames)
      {
        ::GlobalUnlock(hDevNames);
        ::GlobalFree(hDevNames);
        hDevNames = 0;
        DevNames = 0;
      }
  }

const char far* DlgReportSetup::ReportData::GetDriverName() const
  {
    return DevNames ? (char far*)DevNames + DevNames->wDriverOffset : 0;
  }

const char far* DlgReportSetup::ReportData::GetDeviceName() const
  {
    return DevNames ? (char far*)DevNames + DevNames->wDeviceOffset : 0;
  }

const char far* DlgReportSetup::ReportData::GetOutputName() const
  {
    return DevNames ? (char far*)DevNames + DevNames->wOutputOffset : 0;
  }

void DlgReportSetup::ReportData::SetDevNames(const char far* driver,
                                             const char far* device,
                                             const char far* output)
  {
    ClearDevNames();
    if (!driver || !device || !output)
      return;

    int size1 = strlen(driver)+1;
    int size2 = strlen(device)+1;
    int size3 = strlen(output)+1;
    hDevNames = ::GlobalAlloc(GHND, sizeof(DEVNAMES)+size1+size2+size3);
    DevNames = (DEVNAMES far*)::GlobalLock(hDevNames);
    char far* p = (char far*)(DevNames + 1);

    DevNames->wDriverOffset = WORD((char near*)p - (char near*)DevNames);
    if (driver)
      while (*driver)
        *p++ = *driver++;
    *p++ = 0;

    DevNames->wDeviceOffset = WORD((char near*)p - (char near*)DevNames);
    if (device)
      while (*device)
        *p++ = *device++;
    *p++ = 0;

    DevNames->wOutputOffset = WORD((char near*)p - (char near*)DevNames);
    if (output)
      while (*output)
        *p++ = *output++;
    *p++ = 0;

    DevNames->wDefault = FALSE;
  }

TPrintDC* DlgReportSetup::ReportData::TransferDC()
  {
    if (!hDC)
      return 0;
    HDC dc = hDC;
    hDC = 0;
    return new TPrintDC (dc, AutoDelete);
  };

